import scipy.stats as stats
from time import sleep
import json

import matplotlib.pyplot as plt
import datetime
from PIL import Image, ImageDraw, ImageFont

import numpy as np

parsed = json.loads(open('data.json').read())

standingstable = parsed['props']['pageProps']['initialMedals']['medalStandings']['medalsTable']

ranks = [next((x for x in cs['medalsNumber'] if x['type'] == 'Total'), None) for cs in standingstable]

# these have the same order, so index of country in countries aligns with index of medals in medals
countries = [country['description'] for country in standingstable]
medals = [(r['gold'], r['silver'], r['bronze']) for r in ranks]

# let the x-axis be the gold to silver multiplier (1 gold = 1 silver -> inf silver)
# let the y-axis be the silver to bronze multiplier (1 silver = 1 bronze -> inf bronze)

# x and y will be log scales for 1 to 100
# so the exponent goes from 0 to 2
# with 100 steps

steps = 1000
x = np.logspace(0, 5, num=steps, base=2)
y = np.logspace(0, 5, num=steps, base=2)
print(x)

# for every country, calculate total points at each x,y point
adjusted_points = [np.array([c[0] + (c[1] / xx) + (c[2] / (xx * yy)) for yy in y for xx in x]).reshape(steps, steps) for c in medals]
# print(f'number of countries: {len(medals)}')
# print(f'size of adjusted_points: {np.size(adjusted_points)}')
all_points = np.stack(adjusted_points, axis=0)
# print(f'size of all_points: {np.size(all_points)}')



#x, y, are medal ratios, z is country, value at (xyz) is country;s rank at that ratio
overall_ranks = stats.rankdata(all_points * -1, method='dense', axis=0)



X, Y = np.meshgrid(x, y)

max_rank = overall_ranks.max()

ticks_to_label = x[::int(steps/10)]
ticks_to_label_y = y[::int(steps/10)]
dist_btw_ticks = steps/(len(ticks_to_label))
ticks_step_distribution = np.arange(1, steps, dist_btw_ticks)
print(ticks_to_label)
for country_idx in range(len(countries)):
  z=np.array(overall_ranks[country_idx])
  Z = z.reshape(steps, steps)
  plt.figure()
  plt.title(countries[country_idx])
  plt.xlabel('Silver per Gold')
  plt.ylabel('Bronze per Silver')

  plt.contourf(np.arange(0, steps), np.arange(0, steps),  Z, vmin=1, vmax=max_rank, cmap='gist_ncar')

  plt.xticks(np.arange(0, steps, dist_btw_ticks), labels=np.round(ticks_to_label, 1))
  plt.yticks(np.arange(0, steps, dist_btw_ticks), labels=np.round(ticks_to_label_y, 1))
  plt.colorbar(ticks=np.arange(overall_ranks[country_idx].min(), overall_ranks[country_idx].max()+1))
  # plt.show()
  plt.savefig(f'countries/{countries[country_idx]}.png', dpi=500, bbox_inches='tight', pad_inches=1)
  plt.close()

# slice a diagonal for when x=y, ie same ratio of value from gold->silver as silver-> bronze (reasonable)
equal_slice = all_points[:, np.arange(steps), np.arange(steps)]
print(equal_slice)



equal_ranks = stats.rankdata(equal_slice * -1, method='dense', axis=0)

plt.figure()
plt.tight_layout()
plt.title('Country Ranks by ratio of medals')
lines = plt.plot(equal_ranks.transpose())
plt.xticks(np.arange(0, steps, dist_btw_ticks), labels=np.round(ticks_to_label, 2))

plt.legend(countries, bbox_to_anchor=(1, 1.05), fancybox=True)
plt.gca().set_ylim([max_rank + 1, 0])
plt.xlabel('Silver Medals per gold')
plt.ylabel('Country Rank')

plt.savefig('Ranks.png', dpi=500, bbox_inches='tight', pad_inches=1)

#####################

plt.figure()
plt.title('Country equivalent gold medals by ratio of medals')
lines = plt.plot(equal_slice.transpose())

plt.xticks(np.arange(0, steps, dist_btw_ticks), labels=np.round(ticks_to_label, 2))
plt.legend(countries, bbox_to_anchor=(1, 1.05), fancybox=True)
plt.ylabel('Equivalent Gold Medals')
plt.xlabel('Silver Medals per gold')
plt.savefig('Golds.png', dpi=500, bbox_inches='tight', pad_inches=1)

################
# normalized by current (x-position) leader

plt.figure()
plt.title('Normalized country equivalent gold medals by ratio of medals')
normed = equal_slice / equal_slice.max(axis=0)
normed = normed.transpose()
print('normed')
print(normed)
lines = plt.plot(normed)
plt.xticks(np.arange(0, steps, dist_btw_ticks), labels=np.round(ticks_to_label, 2))

plt.legend(countries, bbox_to_anchor=(1, 1.05), fancybox=True)
plt.ylabel('Normalized Equivalent Gold Medals')
plt.xlabel('Silver Medals per gold')
plt.savefig('NormGolds.png', dpi=500, bbox_inches='tight', pad_inches=1)

print(countries)

dt = datetime.datetime.now(datetime.timezone.utc) 
utc_time = dt.replace(tzinfo=datetime.timezone.utc) 

img=Image.new("RGBA", (1200,50),(200,200,30))
font = ImageFont.load_default(size=40)
draw = ImageDraw.Draw(img)
draw.text((0, 0),f'Last Plotted UTC: {utc_time}',(0,0,200), font=font)
draw = ImageDraw.Draw(img)
img.save("rendertimestamp.png")


print()